//
// hello.HC-SR501.c
//
// HC-SR501 motion detector hello-world
//    9600 baud FTDI interface
//
// Neil Gershenfeld 11/16/15
// (c) Massachusetts Institute of Technology 2015
//
// This work may be reproduced, modified, distributed,
// performed, and displayed for any purpose. Copyright is
// retained and must be preserved. The work is provided
// as is; no warranty is provided, and users accept all
// liability.
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "core.h"

#define serial_port PORTB
#define serial_direction DDRB
#define serial_pins PINB
#define serial_pin_in (1 << PB1)
#define serial_pin_out (1 << PB2)
#define serial_interrupt (1 << PCIE0)
#define serial_interrupt_pin (1 << PCINT0)

#define sensor_pin (1 << PB3)

static char is_strum = 0;  // 0 for false, 1 for true
static char high = 0;
static char low = 0;

ISR(PCINT0_vect) {
  // pin change interrupt handler
  static char chr;

  get_char(&serial_pins, serial_pin_in, &chr);
  if (chr == node_id) {
    // send if a strum has happened and reset
    output(serial_direction, serial_pin_out);
    put_char(&serial_port, serial_pin_out, is_strum);
//    char_delay();
    put_char(&serial_port, serial_pin_out, low);
//    char_delay();
    put_char(&serial_port, serial_pin_out, high);
    is_strum = 0;
    input(serial_direction, serial_pin_out);
  }
}

int run(void) {
  // main
  static char chrl;
  static char chrh;

  // set clock divider to /1
  CLKPR = (1 << CLKPCE);
  CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);

  // initialize serial pin
  set(serial_port, serial_pin_out);
  input(serial_direction, serial_pin_out);

  // init A/D
  ADMUX = (0 << REFS2) | (0 << REFS1) | (0 << REFS0) // Vcc ref
          | (0 << ADLAR) // right adjust
          | (0 << MUX3) | (0 << MUX2) | (1 << MUX1) | (1 << MUX0); // ADC3
  ADCSRA = (1 << ADEN) // enable
           | (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); // prescaler /128

  // set up pin change interrupt on input pin
  GIMSK = 0b00100000;    // turns on pin change interrupts
  PCMSK = 0b00010011;    // turn on interrupts on pins PB0, PB1, &amp;amp; PB4
//  set(GIMSK, serial_interrupt);
//  set(PCMSK0, serial_interrupt_pin);
  sei();

  // main loop
  while (1) {
    // initiate conversion
    ADCSRA |= (1 << ADSC);

    // wait for completion voltage analog -> digital
    while (ADCSRA & (1 << ADSC));
    chrl = ADCL;
    chrh = ADCH;

    int value = 256 * chrh + (unsigned char)chrl;

    if (value < LOW_THRESHOLD|| value > HIGH_THRESHOLD) {
      is_strum = 1;
      high = chrh;
      low = chrl;
    } else {
      is_strum = 0;
    }
  }
}

//int main(void) {
//  run();
//}
